@@ -5,7 +5,7 @@
 ** Given an image  and block number, identify which inode it is used by
 ** 
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
-** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
+** Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
 **
 ** TASK
@@ -200,7 +200,7 @@ main(int argc, char **argv1)
             {
                 TSK_TCHAR envstr[32];
                 TSNPRINTF(envstr, 32, _TSK_T("TZ=%s"), OPTARG);
-                if (0 != PUTENV(envstr)) {
+                if (0 != TPUTENV(envstr)) {
                     tsk_fprintf(stderr, "error setting environment");
                     exit(1);
                 }
@@ -247,7 +247,7 @@ main(int argc, char **argv1)
 
     if ((fs = tsk_fs_open_img(img, imgaddr * img->sector_size, fstype)) == NULL) {
         tsk_error_print(stderr);
-        if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+        if (tsk_error_get_errno() == TSK_ERR_FS_UNSUPTYPE)
             tsk_fs_type_print(stderr);
         img->close(img);
         if (path)
