@@ -2,7 +2,7 @@
 ** The Sleuth Kit
 **
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
-** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
+** Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
@@ -89,9 +89,9 @@ print_block(const TSK_FS_BLOCK * fs_block, void *ptr)
     if (fwrite(fs_block->buf, fs_block->fs_info->block_size, 1,
             stdout) != 1) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_WRITE;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "blkls_lib: error writing to stdout: %s", strerror(errno));
+        tsk_error_set_errno(TSK_ERR_FS_WRITE);
+        tsk_error_set_errstr("blkls_lib: error writing to stdout: %s",
+            strerror(errno));
         return TSK_WALK_ERROR;
     }
 
@@ -129,9 +129,9 @@ slack_file_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr,
     else if (data->flen == 0) {
         if (fwrite(buf, size, 1, stdout) != 1) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_WRITE;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "blkls_lib: error writing to stdout: %s", strerror(errno));
+            tsk_error_set_errno(TSK_ERR_FS_WRITE);
+            tsk_error_set_errstr("blkls_lib: error writing to stdout: %s",
+                strerror(errno));
             return TSK_WALK_ERROR;
         }
     }
@@ -141,9 +141,9 @@ slack_file_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr,
         memset(buf, 0, (size_t) data->flen);
         if (fwrite(buf, size, 1, stdout) != 1) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_WRITE;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "blkls_lib: error writing to stdout: %s", strerror(errno));
+            tsk_error_set_errno(TSK_ERR_FS_WRITE);
+            tsk_error_set_errstr("blkls_lib: error writing to stdout: %s",
+                strerror(errno));
             return TSK_WALK_ERROR;
         }
         data->flen = 0;
@@ -234,9 +234,9 @@ tsk_fs_blkls(TSK_FS_INFO * fs, TSK_FS_BLKLS_FLAG_ENUM a_blklsflags,
 #ifdef TSK_WIN32
         if (-1 == _setmode(_fileno(stdout), _O_BINARY)) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_WRITE;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "blkls_lib: error setting stdout to binary: %s",
+            tsk_error_set_errno(TSK_ERR_FS_WRITE);
+            tsk_error_set_errstr
+                ("blkls_lib: error setting stdout to binary: %s",
                 strerror(errno));
             return 1;
         }
