@@ -5,7 +5,7 @@
 ** Content and meta data layer support for the FAT file system 
 **
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
-** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
+** Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
@@ -52,6 +52,8 @@
 /* TTL is 0 if the entry has not been used.  TTL of 1 means it was the
  * most recently used, and TTL of FAT_CACHE_N means it was the least 
  * recently used.  This function has a LRU replacement algo
+ *
+ * Note: This routine assumes &fatfs->cache_lock is locked by the caller.
  */
 // return -1 on error, or cache index on success (0 to FAT_CACHE_N)
 
@@ -105,10 +107,9 @@ getFATCacheIdx(FATFS_INFO * fatfs, TSK_DADDR_T sect)
     if (cnt != FAT_CACHE_B) {
         if (cnt >= 0) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_READ;
+            tsk_error_set_errno(TSK_ERR_FS_READ);
         }
-        snprintf(tsk_errstr2, TSK_ERRSTR_L,
-            "getFATCacheIdx: FAT: %" PRIuDADDR, sect);
+        tsk_error_set_errstr2("getFATCacheIdx: FAT: %" PRIuDADDR, sect);
         return -1;
     }
 
@@ -126,6 +127,7 @@ getFATCacheIdx(FATFS_INFO * fatfs, TSK_DADDR_T sect)
 
     fatfs->fatc_ttl[cidx] = 1;
     fatfs->fatc_addr[cidx] = sect;
+
     return cidx;
 }
 
@@ -165,9 +167,9 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
         }
 
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "fatfs_getFAT: invalid cluster address: %" PRIuDADDR, clust);
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("fatfs_getFAT: invalid cluster address: %"
+            PRIuDADDR, clust);
         return 1;
     }
 
@@ -175,9 +177,9 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
     case TSK_FS_TYPE_FAT12:
         if (clust & 0xf000) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_ARG;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "fatfs_getFAT: TSK_FS_TYPE_FAT12 Cluster %" PRIuDADDR
+            tsk_error_set_errno(TSK_ERR_FS_ARG);
+            tsk_error_set_errstr
+                ("fatfs_getFAT: TSK_FS_TYPE_FAT12 Cluster %" PRIuDADDR
                 " too large", clust);
             return 1;
         }
@@ -186,10 +188,14 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
         sect = fatfs->firstfatsect +
             ((clust + (clust >> 1)) >> fatfs->ssize_sh);
 
+        tsk_take_lock(&fatfs->cache_lock);
+
         /* Load the FAT if we don't have it */
         // see if it is in the cache
-        if (-1 == (cidx = getFATCacheIdx(fatfs, sect)))
+        if (-1 == (cidx = getFATCacheIdx(fatfs, sect))) {
+            tsk_release_lock(&fatfs->cache_lock);
             return 1;
+        }
 
         /* get the offset into the cache */
         offs = ((sect - fatfs->fatc_addr[cidx]) << fatfs->ssize_sh) +
@@ -206,12 +212,13 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
                 tsk_fs_read(fs, sect * fs->block_size,
                 fatfs->fatc_buf[cidx], FAT_CACHE_B);
             if (cnt != FAT_CACHE_B) {
+                tsk_release_lock(&fatfs->cache_lock);
                 if (cnt >= 0) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_FS_READ;
+                    tsk_error_set_errno(TSK_ERR_FS_READ);
                 }
-                snprintf(tsk_errstr2, TSK_ERRSTR_L,
-                    "fatfs_getFAT: TSK_FS_TYPE_FAT12 FAT overlap: %"
+                tsk_error_set_errstr2
+                    ("fatfs_getFAT: TSK_FS_TYPE_FAT12 FAT overlap: %"
                     PRIuDADDR, sect);
                 return 1;
             }
@@ -225,6 +232,8 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
 
         tmp16 = tsk_getu16(fs->endian, a_ptr);
 
+        tsk_release_lock(&fatfs->cache_lock);
+
         /* slide it over if it is one of the odd clusters */
         if (clust & 1)
             tmp16 >>= 4;
@@ -246,8 +255,14 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
     case TSK_FS_TYPE_FAT16:
         /* Get sector in FAT for cluster and load it if needed */
         sect = fatfs->firstfatsect + ((clust << 1) >> fatfs->ssize_sh);
-        if (-1 == (cidx = getFATCacheIdx(fatfs, sect)))
+
+        tsk_take_lock(&fatfs->cache_lock);
+
+        if (-1 == (cidx = getFATCacheIdx(fatfs, sect))) {
+            tsk_release_lock(&fatfs->cache_lock);
             return 1;
+        }
+
 
         /* get pointer to entry in the cache buffer */
         a_ptr = (uint8_t *) fatfs->fatc_buf[cidx] +
@@ -256,6 +271,8 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
 
         *value = tsk_getu16(fs->endian, a_ptr) & FATFS_16_MASK;
 
+        tsk_release_lock(&fatfs->cache_lock);
+
         /* sanity check */
         if ((*value > (fatfs->lastclust)) &&
             (*value < (0x0ffffff7 & FATFS_16_MASK))) {
@@ -270,8 +287,14 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
     case TSK_FS_TYPE_FAT32:
         /* Get sector in FAT for cluster and load if needed */
         sect = fatfs->firstfatsect + ((clust << 2) >> fatfs->ssize_sh);
-        if (-1 == (cidx = getFATCacheIdx(fatfs, sect)))
+
+        tsk_take_lock(&fatfs->cache_lock);
+
+        if (-1 == (cidx = getFATCacheIdx(fatfs, sect))) {
+            tsk_release_lock(&fatfs->cache_lock);
             return 1;
+        }
+
 
         /* get pointer to entry in current buffer */
         a_ptr = (uint8_t *) fatfs->fatc_buf[cidx] +
@@ -280,6 +303,8 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
 
         *value = tsk_getu32(fs->endian, a_ptr) & FATFS_32_MASK;
 
+        tsk_release_lock(&fatfs->cache_lock);
+
         /* sanity check */
         if ((*value > fatfs->lastclust) &&
             (*value < (0x0ffffff7 & FATFS_32_MASK))) {
@@ -294,9 +319,9 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
 
     default:
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "fatfs_getFAT: Unknown FAT type: %d", fatfs->fs_info.ftype);
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("fatfs_getFAT: Unknown FAT type: %d",
+            fatfs->fs_info.ftype);
         return 1;
     }
 }
@@ -413,16 +438,16 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
      */
     if (a_start_blk < fs->first_block || a_start_blk > fs->last_block) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_WALK_RNG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "%s: Start block: %" PRIuDADDR "", myname, a_start_blk);
+        tsk_error_set_errno(TSK_ERR_FS_WALK_RNG);
+        tsk_error_set_errstr("%s: Start block: %" PRIuDADDR "", myname,
+            a_start_blk);
         return 1;
     }
     if (a_end_blk < fs->first_block || a_end_blk > fs->last_block) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_WALK_RNG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "%s: End block: %" PRIuDADDR "", myname, a_end_blk);
+        tsk_error_set_errno(TSK_ERR_FS_WALK_RNG);
+        tsk_error_set_errstr("%s: End block: %" PRIuDADDR "", myname,
+            a_end_blk);
         return 1;
     }
 
@@ -475,11 +500,11 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
             if (cnt != fs->block_size * 8) {
                 if (cnt >= 0) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_FS_READ;
+                    tsk_error_set_errno(TSK_ERR_FS_READ);
                 }
-                snprintf(tsk_errstr2, TSK_ERRSTR_L,
-                    "fatfs_block_walk: pre-data area block: %"
-                    PRIuDADDR, addr);
+                tsk_error_set_errstr2
+                    ("fatfs_block_walk: pre-data area block: %" PRIuDADDR,
+                    addr);
                 free(data_buf);
                 tsk_fs_block_free(fs_block);
                 return 1;
@@ -606,10 +631,10 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
         if (cnt != fs->block_size * read_size) {
             if (cnt >= 0) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_FS_READ;
+                tsk_error_set_errno(TSK_ERR_FS_READ);
             }
-            snprintf(tsk_errstr2, TSK_ERRSTR_L,
-                "fatfs_block_walk: block: %" PRIuDADDR, addr);
+            tsk_error_set_errstr2("fatfs_block_walk: block: %" PRIuDADDR,
+                addr);
             free(data_buf);
             tsk_fs_block_free(fs_block);
             return 1;
@@ -656,9 +681,8 @@ static uint8_t
 fatfs_fscheck(TSK_FS_INFO * fs, FILE * hFile)
 {
     tsk_error_reset();
-    tsk_errno = TSK_ERR_FS_UNSUPFUNC;
-    snprintf(tsk_errstr, TSK_ERRSTR_L,
-        "fscheck not implemented for FAT yet");
+    tsk_error_set_errno(TSK_ERR_FS_UNSUPFUNC);
+    tsk_error_set_errstr("fscheck not implemented for FAT yet");
     return 1;
 
     /* Check that allocated dentries point to start of allcated cluster chain */
@@ -719,10 +743,10 @@ fatfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     if (cnt != fs->block_size) {
         if (cnt >= 0) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_READ;
+            tsk_error_set_errno(TSK_ERR_FS_READ);
         }
-        snprintf(tsk_errstr2, TSK_ERRSTR_L,
-            "fatfs_fsstat: root directory: %" PRIuDADDR, fatfs->rootsect);
+        tsk_error_set_errstr2("fatfs_fsstat: root directory: %" PRIuDADDR,
+            fatfs->rootsect);
         free(data_buf);
         return 1;
     }
@@ -841,10 +865,10 @@ fatfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
             if (cnt != sizeof(fatfs_fsinfo)) {
                 if (cnt >= 0) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_FS_READ;
+                    tsk_error_set_errno(TSK_ERR_FS_READ);
                 }
-                snprintf(tsk_errstr2, TSK_ERRSTR_L,
-                    "fatfs_fsstat: TSK_FS_TYPE_FAT32 FSINFO block: %"
+                tsk_error_set_errstr2
+                    ("fatfs_fsstat: TSK_FS_TYPE_FAT32 FSINFO block: %"
                     PRIuDADDR, (TSK_DADDR_T) tsk_getu16(fs->endian,
                         sb->a.f32.fsinfo));
                 free(data_buf);
@@ -892,8 +916,8 @@ fatfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
         tsk_fprintf(hFile, "** FS Info Sector: %" PRIu16 "\n",
             tsk_getu16(fs->endian, sb->a.f32.fsinfo));
 
-        tsk_fprintf(hFile, "** Backup Boot Sector: %" PRIu32 "\n",
-            tsk_getu32(fs->endian, sb->a.f32.bs_backup));
+        tsk_fprintf(hFile, "** Backup Boot Sector: %" PRIu16 "\n",
+            tsk_getu16(fs->endian, sb->a.f32.bs_backup));
     }
 
     for (i = 0; i < fatfs->numfat; i++) {
@@ -1121,12 +1145,14 @@ fatfs_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
     TSK_FS_META *fs_meta;
     TSK_FS_FILE *fs_file;
     TSK_FS_META_NAME_LIST *fs_name_list;
-    FATFS_INFO *fatfs = (FATFS_INFO *) fs;
     FATFS_PRINT_ADDR print;
+    fatfs_dentry dep;
+    char timeBuf[128];
 
     // clean up any error messages that are lying around
     tsk_error_reset();
 
+
     if ((fs_file = tsk_fs_file_open_meta(fs, NULL, inum)) == NULL) {
         return 1;
     }
@@ -1140,7 +1166,7 @@ fatfs_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
     tsk_fprintf(hFile, "File Attributes: ");
 
     /* This should only be null if we have the root directory or special file */
-    if (fatfs->dep == NULL) {
+    if (fatfs_dinode_load(fs, &dep, inum)) {
         if (inum == FATFS_ROOTINO)
             tsk_fprintf(hFile, "Directory\n");
         else if (fs_file->meta->type == TSK_FS_META_TYPE_VIRT)
@@ -1148,24 +1174,24 @@ fatfs_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
         else
             tsk_fprintf(hFile, "File\n");
     }
-    else if ((fatfs->dep->attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN) {
+    else if ((dep.attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN) {
         tsk_fprintf(hFile, "Long File Name\n");
     }
     else {
-        if (fatfs->dep->attrib & FATFS_ATTR_DIRECTORY)
+        if (dep.attrib & FATFS_ATTR_DIRECTORY)
             tsk_fprintf(hFile, "Directory");
-        else if (fatfs->dep->attrib & FATFS_ATTR_VOLUME)
+        else if (dep.attrib & FATFS_ATTR_VOLUME)
             tsk_fprintf(hFile, "Volume Label");
         else
             tsk_fprintf(hFile, "File");
 
-        if (fatfs->dep->attrib & FATFS_ATTR_READONLY)
+        if (dep.attrib & FATFS_ATTR_READONLY)
             tsk_fprintf(hFile, ", Read Only");
-        if (fatfs->dep->attrib & FATFS_ATTR_HIDDEN)
+        if (dep.attrib & FATFS_ATTR_HIDDEN)
             tsk_fprintf(hFile, ", Hidden");
-        if (fatfs->dep->attrib & FATFS_ATTR_SYSTEM)
+        if (dep.attrib & FATFS_ATTR_SYSTEM)
             tsk_fprintf(hFile, ", System");
-        if (fatfs->dep->attrib & FATFS_ATTR_ARCHIVE)
+        if (dep.attrib & FATFS_ATTR_ARCHIVE)
             tsk_fprintf(hFile, ", Archive");
 
         tsk_fprintf(hFile, "\n");
@@ -1180,26 +1206,39 @@ fatfs_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
 
     if (sec_skew != 0) {
         tsk_fprintf(hFile, "\nAdjusted Directory Entry Times:\n");
-        fs_meta->mtime -= sec_skew;
-        fs_meta->atime -= sec_skew;
-        fs_meta->crtime -= sec_skew;
 
-        tsk_fprintf(hFile, "Written:\t%s", ctime(&fs_meta->mtime));
-        tsk_fprintf(hFile, "Accessed:\t%s", ctime(&fs_meta->atime));
-        tsk_fprintf(hFile, "Created:\t%s", ctime(&fs_meta->crtime));
+        if (fs_meta->mtime)
+            fs_meta->mtime -= sec_skew;
+        if (fs_meta->atime)
+            fs_meta->atime -= sec_skew;
+        if (fs_meta->crtime)
+            fs_meta->crtime -= sec_skew;
 
-        fs_meta->mtime += sec_skew;
-        fs_meta->atime += sec_skew;
-        fs_meta->crtime += sec_skew;
+        tsk_fprintf(hFile, "Written:\t%s\n",
+            tsk_fs_time_to_str(fs_meta->mtime, timeBuf));
+        tsk_fprintf(hFile, "Accessed:\t%s\n",
+            tsk_fs_time_to_str(fs_meta->atime, timeBuf));
+        tsk_fprintf(hFile, "Created:\t%s\n",
+            tsk_fs_time_to_str(fs_meta->crtime, timeBuf));
+
+        if (fs_meta->mtime == 0)
+            fs_meta->mtime += sec_skew;
+        if (fs_meta->atime == 0)
+            fs_meta->atime += sec_skew;
+        if (fs_meta->crtime == 0)
+            fs_meta->crtime += sec_skew;
 
         tsk_fprintf(hFile, "\nOriginal Directory Entry Times:\n");
     }
     else
         tsk_fprintf(hFile, "\nDirectory Entry Times:\n");
 
-    tsk_fprintf(hFile, "Written:\t%s", ctime(&fs_meta->mtime));
-    tsk_fprintf(hFile, "Accessed:\t%s", ctime(&fs_meta->atime));
-    tsk_fprintf(hFile, "Created:\t%s", ctime(&fs_meta->crtime));
+    tsk_fprintf(hFile, "Written:\t%s\n", tsk_fs_time_to_str(fs_meta->mtime,
+            timeBuf));
+    tsk_fprintf(hFile, "Accessed:\t%s\n",
+        tsk_fs_time_to_str(fs_meta->atime, timeBuf));
+    tsk_fprintf(hFile, "Created:\t%s\n",
+        tsk_fs_time_to_str(fs_meta->crtime, timeBuf));
 
     tsk_fprintf(hFile, "\nSectors:\n");
 
@@ -1232,8 +1271,8 @@ uint8_t
 fatfs_jopen(TSK_FS_INFO * fs, TSK_INUM_T inum)
 {
     tsk_error_reset();
-    tsk_errno = TSK_ERR_FS_UNSUPFUNC;
-    snprintf(tsk_errstr, TSK_ERRSTR_L, "FAT does not have a journal\n");
+    tsk_error_set_errno(TSK_ERR_FS_UNSUPFUNC);
+    tsk_error_set_errstr("FAT does not have a journal\n");
     return 1;
 }
 
@@ -1243,8 +1282,8 @@ fatfs_jentry_walk(TSK_FS_INFO * fs, int a_flags,
     TSK_FS_JENTRY_WALK_CB a_action, void *a_ptr)
 {
     tsk_error_reset();
-    tsk_errno = TSK_ERR_FS_UNSUPFUNC;
-    snprintf(tsk_errstr, TSK_ERRSTR_L, "FAT does not have a journal\n");
+    tsk_error_set_errno(TSK_ERR_FS_UNSUPFUNC);
+    tsk_error_set_errstr("FAT does not have a journal\n");
     return 1;
 }
 
@@ -1255,8 +1294,8 @@ fatfs_jblk_walk(TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end,
     int a_flags, TSK_FS_JBLK_WALK_CB a_action, void *a_ptr)
 {
     tsk_error_reset();
-    tsk_errno = TSK_ERR_FS_UNSUPFUNC;
-    snprintf(tsk_errstr, TSK_ERRSTR_L, "FAT does not have a journal\n");
+    tsk_error_set_errno(TSK_ERR_FS_UNSUPFUNC);
+    tsk_error_set_errstr("FAT does not have a journal\n");
     return 1;
 }
 
@@ -1273,17 +1312,15 @@ fatfs_close(TSK_FS_INFO * fs)
     FATFS_INFO *fatfs = (FATFS_INFO *) fs;
     fs->tag = 0;
 
-    free(fatfs->dinodes);
-
     if (fatfs->dir_buf)
         free(fatfs->dir_buf);
     if (fatfs->par_buf)
         free(fatfs->par_buf);
 
-    tsk_list_free(fs->list_inum_named);
-    fs->list_inum_named = NULL;
     free(fatfs->sb);
-    free(fs);
+    tsk_deinit_lock(&fatfs->cache_lock);
+    tsk_deinit_lock(&fatfs->dir_lock);
+    tsk_fs_free(fs);
 }
 
 
@@ -1309,19 +1346,19 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
     TSK_DADDR_T sectors;
     ssize_t cnt;
     int i;
-    uint8_t used_backup_boot = 0;  // set to 1 if we used the backup boot sector
+    uint8_t used_backup_boot = 0;       // set to 1 if we used the backup boot sector
 
     // clean up any error messages that are lying around
     tsk_error_reset();
 
     if (TSK_FS_TYPE_ISFAT(ftype) == 0) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L, "%s: Invalid FS Type", myname);
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("%s: Invalid FS Type", myname);
         return NULL;
     }
 
-    if ((fatfs = (FATFS_INFO *) tsk_malloc(sizeof(*fatfs))) == NULL)
+    if ((fatfs = (FATFS_INFO *) tsk_fs_malloc(sizeof(*fatfs))) == NULL)
         return NULL;
 
     fs = &(fatfs->fs_info);
@@ -1357,9 +1394,9 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         if (cnt != len) {
             if (cnt >= 0) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_FS_READ;
+                tsk_error_set_errno(TSK_ERR_FS_READ);
             }
-            snprintf(tsk_errstr2, TSK_ERRSTR_L, "%s: boot sector", myname);
+            tsk_error_set_errstr2("%s: boot sector", myname);
             fs->tag = 0;
             free(fatfs->sb);
             free(fatfs);
@@ -1379,16 +1416,21 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
                 free(fatsb);
                 free(fatfs);
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_FS_MAGIC;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
-                    "Not a FATFS file system (magic)");
+                tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+                tsk_error_set_errstr("Not a FATFS file system (magic)");
+                if (tsk_verbose)
+                    fprintf(stderr, "Not a FATFS file system (magic)");
                 return NULL;
             }
         }
         // found the magic
         else {
-            if (sb_off)
+            if (sb_off) {
                 used_backup_boot = 1;
+                if (tsk_verbose)
+                    fprintf(stderr,
+                        "fatfs_open: Using backup boot sector\n");
+            }
             break;
         }
     }
@@ -1412,10 +1454,13 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
     }
     else {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Error: sector size (%d) is not a multiple of device size (%d)\nDo you have a disk image instead of a partition image?",
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr
+            ("Error: sector size (%d) is not a multiple of device size (%d)\nDo you have a disk image instead of a partition image?",
             fatfs->ssize, fs->dev_bsize);
+        if (tsk_verbose)
+            fprintf(stderr, "fatfs_open: Invalid sector size (%d)\n",
+                fatfs->ssize);
         fs->tag = 0;
         free(fatsb);
         free(fatfs);
@@ -1431,26 +1476,30 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         (fatfs->csize != 0x10) &&
         (fatfs->csize != 0x20) &&
         (fatfs->csize != 0x40) && (fatfs->csize != 0x80)) {
+        if (tsk_verbose)
+            fprintf(stderr, "fatfs_open: Invalid cluster size (%d)\n",
+                fatfs->csize);
         fs->tag = 0;
         free(fatsb);
         free(fatfs);
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Not a FATFS file system (cluster size)");
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr("Not a FATFS file system (cluster size)");
         return NULL;
     }
 
     // number of FAT tables
     fatfs->numfat = fatsb->numfat;
     if ((fatfs->numfat == 0) || (fatfs->numfat > 8)) {
+        if (tsk_verbose)
+            fprintf(stderr, "fatfs_open: Invalid number of FATS (%d)\n",
+                fatfs->numfat);
         fs->tag = 0;
         free(fatsb);
         free(fatfs);
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Not a FATFS file system (number of FATs)");
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr("Not a FATFS file system (number of FATs)");
         return NULL;
     }
 
@@ -1470,23 +1519,31 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
             tsk_getu32(fs->endian, fatsb->a.f32.sectperfat32);
 
     if (fatfs->sectperfat == 0) {
+        if (tsk_verbose)
+            fprintf(stderr,
+                "fatfs_open: Invalid number of sectors per FAT (%d)\n",
+                fatfs->sectperfat);
         fs->tag = 0;
         free(fatsb);
         free(fatfs);
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Not a FATFS file system (invalid sectors per FAT)");
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr
+            ("Not a FATFS file system (invalid sectors per FAT)");
         return NULL;
     }
 
     fatfs->firstfatsect = tsk_getu16(fs->endian, fatsb->reserved);
     if ((fatfs->firstfatsect == 0) || (fatfs->firstfatsect > sectors)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_WALK_RNG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Not a FATFS file system (invalid first FAT sector %"
+        tsk_error_set_errno(TSK_ERR_FS_WALK_RNG);
+        tsk_error_set_errstr
+            ("Not a FATFS file system (invalid first FAT sector %"
             PRIuDADDR ")", fatfs->firstfatsect);
+        if (tsk_verbose)
+            fprintf(stderr,
+                "fatfs_open: Invalid first FAT (%" PRIuDADDR ")\n",
+                fatfs->firstfatsect);
 
         fs->tag = 0;
         free(fatsb);
@@ -1552,9 +1609,12 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
             free(fatsb);
             free(fatfs);
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_MAGIC;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "Too many sectors for TSK_FS_TYPE_FAT12: try auto-detect mode");
+            tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+            tsk_error_set_errstr
+                ("Too many sectors for TSK_FS_TYPE_FAT12: try auto-detect mode");
+            if (tsk_verbose)
+                fprintf(stderr,
+                    "fatfs_open: Too many sectors for FAT12\n");
             return NULL;
         }
     }
@@ -1564,9 +1624,11 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         free(fatsb);
         free(fatfs);
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Invalid TSK_FS_TYPE_FAT32 image (numroot != 0)");
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr
+            ("Invalid TSK_FS_TYPE_FAT32 image (numroot != 0)");
+        if (tsk_verbose)
+            fprintf(stderr, "fatfs_open: numroom != 0 for FAT32\n");
         return NULL;
     }
 
@@ -1575,12 +1637,14 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         free(fatsb);
         free(fatfs);
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Invalid FAT image (numroot == 0, and not TSK_FS_TYPE_FAT32)");
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr
+            ("Invalid FAT image (numroot == 0, and not TSK_FS_TYPE_FAT32)");
+        if (tsk_verbose)
+            fprintf(stderr, "fatfs_open: numroom == 0 and not FAT32\n");
         return NULL;
     }
-    
+
     /* additional sanity checks if we think we are using the backup boot sector.
      * The scenario to prevent here is if fat_open is called 6 sectors before the real start
      * of the file system, then we want to detect that it was not a backup that we saw.  
@@ -1592,9 +1656,12 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
             free(fatsb);
             free(fatfs);
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_MAGIC;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                     "Invalid FAT image (Used what we thought was a backup boot sector, but it is not TSK_FS_TYPE_FAT32)");
+            tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+            tsk_error_set_errstr
+                ("Invalid FAT image (Used what we thought was a backup boot sector, but it is not TSK_FS_TYPE_FAT32)");
+            if (tsk_verbose)
+                fprintf(stderr,
+                    "fatfs_open: Had to use backup boot sector, but this isn't FAT32\n");
             return NULL;
         }
         if (fatfs->numroot > 1) {
@@ -1602,33 +1669,38 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
             uint8_t buf2[512];
             int i2;
             int numDiffs;
-            
-            cnt = tsk_fs_read(fs, fatfs->firstfatsect * fatfs->ssize, (char *) buf1, 512);
+
+            cnt =
+                tsk_fs_read(fs, fatfs->firstfatsect * fatfs->ssize,
+                (char *) buf1, 512);
             if (cnt != 512) {
                 if (cnt >= 0) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_FS_READ;
+                    tsk_error_set_errno(TSK_ERR_FS_READ);
                 }
-                snprintf(tsk_errstr2, TSK_ERRSTR_L, "%s: FAT1", myname);
+                tsk_error_set_errstr2("%s: FAT1", myname);
                 fs->tag = 0;
                 free(fatfs->sb);
                 free(fatfs);
                 return NULL;
             }
-            
-            cnt = tsk_fs_read(fs, (fatfs->firstfatsect + fatfs->sectperfat)*fatfs->ssize, (char *) buf2, 512);
+
+            cnt =
+                tsk_fs_read(fs,
+                (fatfs->firstfatsect + fatfs->sectperfat) * fatfs->ssize,
+                (char *) buf2, 512);
             if (cnt != 512) {
                 if (cnt >= 0) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_FS_READ;
+                    tsk_error_set_errno(TSK_ERR_FS_READ);
                 }
-                snprintf(tsk_errstr2, TSK_ERRSTR_L, "%s: FAT2", myname);
+                tsk_error_set_errstr2("%s: FAT2", myname);
                 fs->tag = 0;
                 free(fatfs->sb);
                 free(fatfs);
                 return NULL;
             }
-            
+
             numDiffs = 0;
             for (i2 = 0; i2 < 512; i2++) {
                 if (buf1[i2] != buf2[i2]) {
@@ -1640,14 +1712,19 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
                 free(fatsb);
                 free(fatfs);
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_FS_MAGIC;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
-                         "Invalid FAT image (Too many differences between FATS from guessing (%d diffs))", numDiffs);
+                tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+                tsk_error_set_errstr
+                    ("Invalid FAT image (Too many differences between FATS from guessing (%d diffs))",
+                    numDiffs);
+                if (tsk_verbose)
+                    fprintf(stderr,
+                        "fatfs_open: Too many differences in FAT from guessing (%d diffs)\n",
+                        numDiffs);
                 return NULL;
             }
         }
     }
-    
+
 
     /* Set the mask to use on the cluster values */
     if (ftype == TSK_FS_TYPE_FAT12) {
@@ -1664,9 +1741,9 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         free(fatsb);
         free(fatfs);
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Unknown FAT type in fatfs_open: %d\n", ftype);
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("Unknown FAT type in fatfs_open: %d\n",
+            ftype);
         return NULL;
     }
     fs->duname = "Sector";
@@ -1685,16 +1762,6 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         fatfs->fatc_ttl[i] = 0;
     }
 
-    /* allocate a cluster-sized buffer for inodes */
-    if ((fatfs->dinodes = (char *)
-            tsk_malloc(fatfs->csize << fatfs->ssize_sh)) == NULL) {
-        fs->tag = 0;
-        free(fatsb);
-        free(fatfs);
-        return NULL;
-    }
-
-
     /*
      * block calculations : although there are no blocks in fat, we will
      * use these fields for sector calculations
@@ -1765,7 +1832,8 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
     fs->journ_inum = 0;
 
     // initialize the caches
-    fs->list_inum_named = NULL;
+    tsk_init_lock(&fatfs->cache_lock);
+    tsk_init_lock(&fatfs->dir_lock);
 
     return (fs);
 }
