@@ -7,7 +7,7 @@
 ** depending on the file system type.
 **
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
-** Copyright (c) 2006-2008 Brian Carrier.  All Rights reserved
+** Copyright (c) 2006-2011 Brian Carrier.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
@@ -219,7 +219,8 @@ tsk_fs_name_copy(TSK_FS_NAME * a_fs_name_to,
  * @param a_len Length of buffer
  */
 uint8_t
-tsk_fs_meta_make_ls(TSK_FS_META * a_fs_meta, char *a_buf, size_t a_len)
+tsk_fs_meta_make_ls(const TSK_FS_META * a_fs_meta, char *a_buf,
+    size_t a_len)
 {
     if (a_len < 12) {
         return 1;
@@ -280,25 +281,49 @@ tsk_fs_meta_make_ls(TSK_FS_META * a_fs_meta, char *a_buf, size_t a_len)
 }
 
 
-static void
-tsk_fs_print_time(FILE * hFile, time_t time)
+/** \ingroup fslib
+ * Converts a time value to a string representation. Prints
+ * all zero values instead of 1970 if time is 0.
+ * @param time Time to be displayed.
+ * @param buf Buffer to print into (must be 128 byes or larger)
+ * @returns Pointer to buffer that was passed in.
+ */
+char *
+tsk_fs_time_to_str(time_t time, char buf[128])
 {
+    buf[0] = '\0';
     if (time <= 0) {
-        tsk_fprintf(hFile, "0000-00-00 00:00:00 (UTC)");
+        strncpy(buf, "0000-00-00 00:00:00 (UTC)", 128);
     }
     else {
         struct tm *tmTime = localtime(&time);
 
-        tsk_fprintf(hFile, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d (%s)",
+        snprintf(buf, 128, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d (%s)",
             (int) tmTime->tm_year + 1900,
             (int) tmTime->tm_mon + 1, (int) tmTime->tm_mday,
             tmTime->tm_hour,
             (int) tmTime->tm_min, (int) tmTime->tm_sec,
             tzname[(tmTime->tm_isdst == 0) ? 0 : 1]);
     }
+    return buf;
 }
 
 
+static void
+tsk_fs_print_time(FILE * hFile, time_t time)
+{
+    char foo[128];
+    tsk_fs_time_to_str(time, foo);
+    tsk_fprintf(hFile, "%s", foo);
+}
+
+
+// @@@ We could merge this with the tsk_fs_time_to_str in
+// the future when the feature to include time resolution
+// is added to TSK_FS_META (and then that value would be 
+// passed in and tsk_fs_time_to_str would decide what to
+// round up/down to
+
 /**
  * The only difference with this one is that the time is always
  * 00:00:00, which is applicable for the A-Time in FAT, which does
@@ -620,22 +645,26 @@ tsk_fs_name_print_mac(FILE * hFile, const TSK_FS_FILE * fs_file,
 
         /* atime, mtime, ctime, crtime */
         if (fs_file->meta->atime)
-            tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->atime - time_skew);
+            tsk_fprintf(hFile, "%" PRIu32 "|",
+                fs_file->meta->atime - time_skew);
         else
             tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->atime);
 
         if (fs_file->meta->mtime)
-            tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->mtime - time_skew);
+            tsk_fprintf(hFile, "%" PRIu32 "|",
+                fs_file->meta->mtime - time_skew);
         else
             tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->mtime);
-        
+
         if (fs_file->meta->ctime)
-            tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->ctime - time_skew);
+            tsk_fprintf(hFile, "%" PRIu32 "|",
+                fs_file->meta->ctime - time_skew);
         else
             tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->ctime);
-        
+
         if (fs_file->meta->crtime)
-            tsk_fprintf(hFile, "%" PRIu32 "\n", fs_file->meta->crtime - time_skew);
+            tsk_fprintf(hFile, "%" PRIu32 "\n",
+                fs_file->meta->crtime - time_skew);
         else
             tsk_fprintf(hFile, "%" PRIu32 "\n", fs_file->meta->crtime);
     }
